import { ref } from 'vue';
import { defineStore } from 'pinia';

type CommonCacheType = string;

type FetchApiType = (params: any) => Promise<any>;

export const useCommonCacheStore = defineStore('commonCache', () => {
  const commonCacheMap = ref(new Map<string, any>());
  const commonCacheValueLabelMap = ref(new Map<string, Map<any, string>>());
  const commonCachePendingMap = ref(new Map<string, boolean>());

  const commonCacheFetch = ref(new Map<string, FetchApiType>());

  /**
   *
   * 请求通用缓存数据
   * @param {(commonCacheType | commonCacheType[])} commonCacheType 请求的缓存键值
   * @param {FetchApiType} fetchApi 请求函数
   * @return {*}
   */
  const fetchCommonCache = (
    commonCacheType: CommonCacheType | CommonCacheType[],
    fetchApi: FetchApiType,
  ) => {
    const commonCacheTypes = Array.isArray(commonCacheType) ? commonCacheType : [commonCacheType];
    const promises = commonCacheTypes.map(async (type) => {
      if (commonCacheMap.value.has(type) && !commonCachePendingMap.value.has(type)) {
        return commonCacheMap.value.get(type)!;
      }

      commonCacheMap.value.set(type, []);
      commonCachePendingMap.value.set(type, true);
      const res = await fetchApi({ type }).finally(() => commonCachePendingMap.value.delete(type));
      commonCacheFetch.value.set(type, fetchApi);
      commonCacheMap.value.set(type, res);
      commonCacheValueLabelMap.value.set(
        type,
        new Map(res.map((item) => [item.value, item.label])),
      );
      return res;
    });
    return Promise.all(promises);
  };

  const commonCacheData = new Proxy({} as Record<CommonCacheType, any>, {
    get(_, prop: CommonCacheType) {
      if (typeof prop === 'string' && prop.startsWith('__v_')) {
        // console.trace('get', prop);
        return;
      }
      if (commonCacheMap.value.has(prop)) {
        return commonCacheMap.value.get(prop);
      }
      if (!commonCachePendingMap.value.has(prop)) {
        const fetchCommonCache = commonCacheFetch.value.get(prop);
        fetchCommonCache && fetchCommonCache(prop);
      }
      return commonCacheMap.value.get(prop) || [];
    },
  });

  const showcommonCacheLabel = (commonCacheType: CommonCacheType, code: any) => {
    commonCacheData[commonCacheType];
    return commonCacheValueLabelMap.value.get(commonCacheType)?.get(code) || [];
  };

  const commonCachePending = new Proxy({} as Record<CommonCacheType, boolean>, {
    get(_, prop: CommonCacheType) {
      return commonCachePendingMap.value.get(prop) || false;
    },
  });

  return { commonCacheData, commonCachePending, fetchCommonCache, showcommonCacheLabel };
});
